/***************************************************************************************************
*                                Sample for the usage of class TopoART                             *
*                             Created by Marko Tscherepanow, 23 July 2011                          *
***************************************************************************************************/

// Compile: mcs /r:LibTopoART.dll TopoART_sample1.cs
// Run: mono TopoART_sample1.exe

using System;
using System.IO;
using System.Text.RegularExpressions;
using LibTopoART;
using System.Globalization;

namespace LibTopoART_samples
{
	/// <summary>
	/// Simple TopoART sample. 
	/// <para>First, a datased comprised of 10 samples is learned by a TopoART network. Afterwards,
	/// the training samples are slightly modified by random values and used for predicting cluster
	/// labels.</para>
	/// </summary>
	class TopoART_sample1
	{
		static void Main() {

			// Dataset comprising two clusters, each containing 5 samples
			// Cluster 1: samples 1 to 5
			// Cluster 2: samples 6 to 10
			string Dataset			=	"data/data.txt";		
			long SampleNumber		=	10;
			long InputDimension		=	2;

			decimal[][] InputArray 	=	new decimal[SampleNumber][];
			StreamReader DatasetFile;
			long i;
			string Line;
			ITopoART t;

			// Choose TopoART implementation (false = TopoART; true = Fast_TopoART). If true, class Fast_TopoART will be used 
			// rather than class TopoART. As a result, the computation is accelerated but less accurate. From
			// an algorithmic point of view, both implementations are equal. But Fast_TopoART uses long rather than decimal
			// for the internal represention of data.
			bool USE_FAST_TOPOART = false;
			
			// Load dataset
			using(DatasetFile = new StreamReader(File.Open(Dataset, FileMode.Open))) {
				for(i = 0; i < SampleNumber; ++i) {
					Line=DatasetFile.ReadLine();
					string[] numbers = Regex.Split(Line, @"\s+");
					InputArray[i] = new decimal[InputDimension]; 
					InputArray[i][0] = Decimal.Parse(numbers[1], NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
					InputArray[i][1] = Decimal.Parse(numbers[2], NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
					Console.WriteLine("Read input " + InputArray[i][0] + "; " + InputArray[i][1]);
				}
			}
			
			// Initialise a TopoART network using rho_a = 0.7 and default values for the remaining parameters
			if(USE_FAST_TOPOART) t = new Fast_TopoART(InputDimension, 2, 0.70m);
			else t = new TopoART(InputDimension, 2, 0.70m);
			
			// Train network
			for(i = 0; i < SampleNumber; ++i) {
				t.Learn(InputArray[i]);
			}
			
			// Determine clusters
			t.ComputeClusterIDs();
			
			// Save network in human readable form
			t.SaveText("TopoART.txt");
			
			// Compute predictions for the randomly modified training data
			// (TopoART module 1 (TopoART b) creates a more detailed representation using 
			// more nodes than TopoART module 0 (TopoART a). But the number of clusters is equal.)
			Random rnd=new Random();
			F2_output[] output;
			decimal	x_deviation,y_deviation;
			for(i = 0; i < SampleNumber; ++i) {
				x_deviation = ((decimal)rnd.Next(-1000, 1000)) / 100000m;
				y_deviation = ((decimal)rnd.Next(-1000, 1000)) / 100000m;
				InputArray[i][0] += x_deviation;
				InputArray[i][1] += y_deviation;
				Console.WriteLine("Compute output for: {0:0.#####}; {1:0.#####}", InputArray[i][0], InputArray[i][1]);
				output = t.GetBMOutput(InputArray[i]);
				Console.WriteLine("TopoART a: node {0}, cluster {1}", output[0].bm_node_ID, output[0].bm_cluster_ID);
				Console.WriteLine("TopoART b: node {0}, cluster {1}", output[1].bm_node_ID, output[1].bm_cluster_ID);
			}
		}
	}
}